@@ -16,54 +16,25 @@
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#include <stdio.h>
-#include <assert.h>
 #include "util.h"
 
 #define XC_GGA_X_2D_B86_MGC      124 /* Becke 86 MGC for 2D systems */
 
-static inline void
-func(const XC(func_type) *p, int order, FLOAT x, 
-     FLOAT *f, FLOAT *dfdx, FLOAT *d2fdx2, FLOAT *d3fdx3)
-{
-  static const FLOAT beta=0.003317, gam=0.008323;
-
-  FLOAT dd, ddp, f1, f2, df1, df2, d2f1, d2f2;
-
-  dd    = 1.0 + gam*x*x;
-
-  f1    = beta/X_FACTOR_C*x*x;
-  f2    = POW(dd, 3.0/4.0);
-  *f    = 1.0 + f1/f2;
-
-  if(order < 1) return; /* nothing else to do */
-
-  df1 = beta/X_FACTOR_C*2.0*x;
-  ddp = gam*2.0*3.0/4.0*f2/dd;
-  df2 = ddp*x;
-
-  *dfdx  = (df1*f2 - f1*df2)/(f2*f2);
-
-  if(order < 2) return; /* nothing else to do */
-
-  d2f1 = beta/X_FACTOR_C*2.0;
-  d2f2 = ddp*(1.0 - 2.0/4.0*gam*x*x/dd);
-
-  *d2fdx2 = (2.0*f1*df2*df2 + d2f1*f2*f2 - f2*(2.0*df1*df2 + f1*d2f2))/(f2*f2*f2);
-}
+#include "maple2c/gga_x_2d_b86_mgc.c"
 
+#define func maple2c_func
 #define XC_DIMENSIONS 2
 #include "work_gga_x.c"
 
-const XC(func_info_type) XC(func_info_gga_x_2d_b86_mgc) = {
+const xc_func_info_type xc_func_info_gga_x_2d_b86_mgc = {
   XC_GGA_X_2D_B86_MGC,
   XC_EXCHANGE,
   "Becke 86 with modified gradient correction for 2D",
   XC_FAMILY_GGA,
   {&xc_ref_Pittalis2009_012503, NULL, NULL, NULL, NULL},
-  XC_FLAGS_2D | XC_FLAGS_HAVE_EXC | XC_FLAGS_HAVE_VXC | XC_FLAGS_HAVE_FXC,
-  1e-32, 1e-32, 0.0, 1e-32,
-  NULL, NULL, NULL,
-  work_gga_x,
-  NULL
+  XC_FLAGS_2D | XC_FLAGS_HAVE_EXC | XC_FLAGS_HAVE_VXC | XC_FLAGS_HAVE_FXC | XC_FLAGS_HAVE_KXC,
+  1e-23,
+  0, NULL, NULL,
+  NULL, NULL, 
+  NULL, work_gga_x, NULL
 };
