@@ -16,34 +16,28 @@
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <assert.h>
 #include "util.h"
 
-#define XC_GGA_X_KT1          145 /* Keal and Tozer version 1             */
-#define XC_GGA_XC_KT2         146 /* Keal and Tozer version 2             */
-
-#define HEADER 3
+#define XC_GGA_X_KT1          145 /* Exchange part of Keal and Tozer version 1 */
+#define XC_GGA_XC_KT1         167 /* Keal and Tozer version 1                  */
+#define XC_GGA_XC_KT2         146 /* Keal and Tozer version 2                  */
 
 typedef struct{
-  FLOAT gamma, delta;
+  double gamma, delta;
 } gga_x_kt_params;
 
-
 static void 
-gga_x_kt_init(XC(func_type) *p)
+gga_x_kt_init(xc_func_type *p)
 {
-
   assert(p!=NULL && p->params == NULL);
   p->params = malloc(sizeof(gga_x_kt_params));
 
-  XC(gga_x_kt_set_params)(p, -0.006, 0.1);
+  xc_gga_x_kt_set_params(p, -0.006, 0.1);
 }
 
 
 void 
-XC(gga_x_kt_set_params)(XC(func_type) *p, FLOAT gamma, FLOAT delta)
+xc_gga_x_kt_set_params(xc_func_type *p, double gamma, double delta)
 {
   gga_x_kt_params *params;
 
@@ -54,72 +48,65 @@ XC(gga_x_kt_set_params)(XC(func_type) *p, FLOAT gamma, FLOAT delta)
   params->delta = delta;
 }
 
+#include "maple2c/gga_x_kt.c"
 
-static inline void 
-func(const XC(func_type) *p, int order, FLOAT x, FLOAT ds,
-     FLOAT *f, FLOAT *dfdx, FLOAT *lvrho)
-{
-  FLOAT dd, n13, n43;
-  gga_x_kt_params *params;
+#define func maple2c_func
+#include "work_gga_c.c"
 
-  assert(p != NULL && p->params != NULL);
-  params = (gga_x_kt_params *) (p->params);
-
-  n13 = CBRT(ds);
-  n43 = ds*n13;
-  dd  = 1.0/(n43 + params->delta);
- 
-  *f = 1.0 - params->gamma/X_FACTOR_C * x*x * n43*dd;
-
-  if(order < 1) return;
-
-  *dfdx  = - params->gamma/X_FACTOR_C * 2.0*x * n43*dd;
-  *lvrho = - params->gamma/X_FACTOR_C * x*x * (4.0/3.0)*n13 * params->delta * dd*dd;
-
-  if(order < 2) return;
-
-  /* to be done */
-}
-
-
-#include "work_gga_x.c"
-
-static void
-gga_xc_kt2_init(XC(func_type) *p)
-{
-  static int   funcs_id  [3] = {XC_LDA_X, XC_GGA_X_KT1, XC_LDA_C_VWN};
-  static FLOAT funcs_coef[3] = {1.07173 - 1.0, 1.0, 0.576727};
-
-  XC(mix_init)(p, 3, funcs_id, funcs_coef);  
-}
-
-
-const XC(func_info_type) XC(func_info_gga_x_kt1) = {
+const xc_func_info_type xc_func_info_gga_x_kt1 = {
   XC_GGA_X_KT1,
   XC_EXCHANGE,
-  "Keal and Tozer, version 1",
+  "Exchange part of Keal and Tozer version 1",
   XC_FAMILY_GGA,
   {&xc_ref_Keal2003_3015, NULL, NULL, NULL, NULL},
-  XC_FLAGS_3D | XC_FLAGS_HAVE_EXC | XC_FLAGS_HAVE_VXC,
-  1e-32, 1e-32, 0.0, 1e-32,
-  gga_x_kt_init, 
-  NULL, NULL,
-  work_gga_x,
-  NULL
+  XC_FLAGS_3D | XC_FLAGS_HAVE_EXC | XC_FLAGS_HAVE_VXC | XC_FLAGS_HAVE_FXC | XC_FLAGS_HAVE_KXC,
+  1e-32,
+  0, NULL, NULL,
+  gga_x_kt_init, NULL, 
+  NULL, work_gga_c, NULL
 };
 
 
-const XC(func_info_type) XC(func_info_gga_xc_kt2) = {
+static void
+gga_xc_kt1_init(xc_func_type *p)
+{
+  static int   funcs_id  [2] = {XC_GGA_X_KT1, XC_LDA_C_VWN};
+  static double funcs_coef[2] = {1.0, 1.0};
+
+  xc_mix_init(p, 2, funcs_id, funcs_coef);  
+}
+
+const xc_func_info_type xc_func_info_gga_xc_kt1 = {
+  XC_GGA_XC_KT1,
+  XC_EXCHANGE_CORRELATION,
+  "Keal and Tozer, version 1",
+  XC_FAMILY_GGA,
+  {&xc_ref_Keal2003_3015, NULL, NULL, NULL, NULL},
+  XC_FLAGS_3D | XC_FLAGS_HAVE_EXC | XC_FLAGS_HAVE_VXC | XC_FLAGS_HAVE_FXC | XC_FLAGS_HAVE_KXC,
+  1e-24,
+  0, NULL, NULL,
+  gga_xc_kt1_init, NULL, 
+  NULL, NULL, NULL
+};
+
+static void
+gga_xc_kt2_init(xc_func_type *p)
+{
+  static int   funcs_id  [3] = {XC_LDA_X, XC_GGA_X_KT1, XC_LDA_C_VWN};
+  static double funcs_coef[3] = {1.07173 - 1.0, 1.0, 0.576727};
+
+  xc_mix_init(p, 3, funcs_id, funcs_coef);  
+}
+
+const xc_func_info_type xc_func_info_gga_xc_kt2 = {
   XC_GGA_XC_KT2,
   XC_EXCHANGE_CORRELATION,
   "Keal and Tozer, version 2",
   XC_FAMILY_GGA,
   {&xc_ref_Keal2003_3015, NULL, NULL, NULL, NULL},
-  XC_FLAGS_3D | XC_FLAGS_HAVE_EXC | XC_FLAGS_HAVE_VXC,
-  1e-32, 1e-32, 0.0, 1e-32,
-  gga_xc_kt2_init, 
-  NULL, NULL, NULL,
-  NULL
+  XC_FLAGS_3D | XC_FLAGS_HAVE_EXC | XC_FLAGS_HAVE_VXC | XC_FLAGS_HAVE_FXC | XC_FLAGS_HAVE_KXC,
+  1e-24,
+  0, NULL, NULL,
+  gga_xc_kt2_init, NULL, 
+  NULL, NULL, NULL
 };
-
-
