@@ -16,15 +16,14 @@
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#include <stdio.h>
-#include <string.h>
 
 #include "xc.h"
 #include "config.h"
+#include <stdio.h>
 
-const char * libxc_version = PACKAGE_VERSION;
+static const char * libxc_version = PACKAGE_VERSION;
 
-void XC(version)(int *major, int *minor, int *micro) {
+void xc_version(int *major, int *minor, int *micro) {
 
   *major = -1;
   *minor = -1;
@@ -33,6 +32,6 @@ void XC(version)(int *major, int *minor, int *micro) {
 
 }
 
-const char *XC(version_string)() {
+const char *xc_version_string() {
   return libxc_version;
 }
