@@ -16,27 +16,22 @@
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <assert.h>
-#include <math.h>
 
 #include "util.h"
 
 static void 
-work_gga_c(const XC(func_type) *p, int np, const FLOAT *rho, const FLOAT *sigma,
-	   FLOAT *zk, FLOAT *vrho, FLOAT *vsigma,
-	   FLOAT *v2rho2, FLOAT *v2rhosigma, FLOAT *v2sigma2,
-	   FLOAT *v3rho3, FLOAT *v3rho2sigma, FLOAT *v3rhosigma2, FLOAT *v3sigma3)
+work_gga_c(const xc_func_type *p, int np, const double *rho, const double *sigma,
+	   double *zk, double *vrho, double *vsigma,
+	   double *v2rho2, double *v2rhosigma, double *v2sigma2,
+	   double *v3rho3, double *v3rho2sigma, double *v3rhosigma2, double *v3sigma3)
 {
-  XC(gga_work_c_t) r;
-  FLOAT min_grad2 = p->info->min_grad*p->info->min_grad;
+  xc_gga_work_c_t r;
+  double min_grad2 = p->dens_threshold * p->dens_threshold;
   int ip;
 
-  FLOAT drs, dxtdn, dxtds, ndzdn[2], dxsdn[2], dxsds[2];;
-  FLOAT d2rs, d2xtdn2, d2xtds2, d2xtdns, d2xsdn2[2], d2xsds2[2], d2xsdns[2];
-  FLOAT d3rs, d3xtdn3, d3xtdn2s, d3xtdns2, d3xtds3, d3xsdn3[2], d3xsdn2s[2], d3xsdns2[2], d3xsds3[2];
+  double drs, dxtdn, dxtds, ndzdn[2], dxsdn[2], dxsds[2];;
+  double d2rs, d2xtdn2, d2xtds2, d2xtdns, d2xsdn2[2], d2xsds2[2], d2xsdns[2];
+  double d3rs, d3xtdn3, d3xtdn2s, d3xtdns2, d3xtds3, d3xsdn3[2], d3xsdn2s[2], d3xsdns2[2], d3xsds3[2];
 
   /* set all elements of r to zero */
   memset(&r, 0, sizeof(r));
@@ -50,17 +45,17 @@ work_gga_c(const XC(func_type) *p, int np, const FLOAT *rho, const FLOAT *sigma,
   if(r.order < 0) return;
 
   for(ip = 0; ip < np; ip++){
-    XC(rho2dzeta)(p->nspin, rho, &(r.dens), &(r.zeta));
-
-    if(r.dens < p->info->min_dens) goto end_ip_loop;
+    xc_rho2dzeta(p->nspin, rho, &(r.dens), &(r.z));
 
+    if(r.dens < p->dens_threshold) goto end_ip_loop;
+    
     r.rs = RS(r.dens);
     if(p->nspin == XC_UNPOLARIZED){
       r.ds[0]  = r.dens/2.0;
       r.ds[1]  = r.ds[0];
 
       r.sigmat = max(min_grad2, sigma[0]);
-      r.xt     = SQRT(r.sigmat)/ POW(r.dens, 4.0/3.0);
+      r.xt     = sqrt(r.sigmat)/ pow(r.dens, 4.0/3.0);
 
       r.sigmas[0] = r.sigmat/4.0;
       r.sigmas[1] = r.sigmas[0];
@@ -69,18 +64,24 @@ work_gga_c(const XC(func_type) *p, int np, const FLOAT *rho, const FLOAT *sigma,
       r.xs[0]  = CBRT(2.0)*r.xt;
       r.xs[1]  = r.xs[0];
     }else{
-      r.ds[0]  = max(p->info->min_dens, rho[0]);
-      r.ds[1]  = max(p->info->min_dens, rho[1]);
+      /* there are lots of derivatives that involve inverse
+         powers of (1 +- z). For these not to give NaN, we
+         must have abs(1 +- z) > DBL_EPSILON                 */
+      if(1.0 + r.z < DBL_EPSILON) r.z = -1.0 + DBL_EPSILON;
+      if(1.0 - r.z < DBL_EPSILON) r.z =  1.0 - DBL_EPSILON;
+
+      r.ds[0]  = max(p->dens_threshold, rho[0]);
+      r.ds[1]  = max(p->dens_threshold, rho[1]);
       
       r.sigmat = max(min_grad2, sigma[0] + 2.0*sigma[1] + sigma[2]);
-      r.xt     = SQRT(r.sigmat)/ POW(r.dens, 4.0/3.0);
+      r.xt     = sqrt(r.sigmat)/ pow(r.dens, 4.0/3.0);
       
       r.sigmas[0] = max(min_grad2, sigma[0]);
       r.sigmas[1] = max(min_grad2, sigma[1]);
       r.sigmas[2] = max(min_grad2, sigma[2]);
 
-      r.xs[0] = SQRT(r.sigmas[0])/POW(r.ds[0], 4.0/3.0);
-      r.xs[1] = SQRT(r.sigmas[2])/POW(r.ds[1], 4.0/3.0);
+      r.xs[0] = sqrt(r.sigmas[0])/pow(r.ds[0], 4.0/3.0);
+      r.xs[1] = sqrt(r.sigmas[2])/pow(r.ds[1], 4.0/3.0);
     }
   
     func(p, &r);
@@ -95,8 +96,8 @@ work_gga_c(const XC(func_type) *p, int np, const FLOAT *rho, const FLOAT *sigma,
     dxtdn = -4.0*r.xt/(3.0*r.dens);
     dxtds = r.xt/(2.0*r.sigmat);
     if(p->nspin == XC_POLARIZED){
-      ndzdn[1] = -(r.zeta + 1.0);
-      ndzdn[0] = -(r.zeta - 1.0);
+      ndzdn[1] = -(r.z + 1.0);
+      ndzdn[0] = -(r.z - 1.0);
 
       dxsdn[1] = -4.0/3.0*r.xs[1]/r.ds[1];
       dxsdn[0] = -4.0/3.0*r.xs[0]/r.ds[0];
@@ -165,12 +166,12 @@ work_gga_c(const XC(func_type) *p, int np, const FLOAT *rho, const FLOAT *sigma,
 
 	  v2rho2[is]  = v2rho2[0];
 
-	  v2rho2[is] += r.dfdxs[s1]*dxsdn[s1] + 
-	    ndzdn[s1]*(r.d2fdrsz*drs + r.d2fdzxt*dxtdn + r.d2fdzxs[s2]*dxsdn[s2]) + 
+	  v2rho2[is] += r.dfdxs[s1]*dxsdn[s1] +
+	    ndzdn[s1]*(r.d2fdrsz*drs + r.d2fdzxt*dxtdn + r.d2fdzxs[s2]*dxsdn[s2]) +
 	    r.dens*(r.d2fdrsxs[s1]*drs*dxsdn[s1] + r.d2fdxtxs[s1]*dxtdn*dxsdn[s1]);
 
-	  v2rho2[is] += r.dfdxs[s2]*dxsdn[s2] + 
-	    ndzdn[s2]*(r.d2fdrsz*drs + r.d2fdzxt*dxtdn + r.d2fdzxs[s1]*dxsdn[s1]) + 
+	  v2rho2[is] += r.dfdxs[s2]*dxsdn[s2] +
+	    ndzdn[s2]*(r.d2fdrsz*drs + r.d2fdzxt*dxtdn + r.d2fdzxs[s1]*dxsdn[s1]) +
 	    r.dens*(r.d2fdrsxs[s2]*drs*dxsdn[s2] + r.d2fdxtxs[s2]*dxtdn*dxsdn[s2]);
 
 	  v2rho2[is] += r.d2fdz2*ndzdn[s1]*ndzdn[s2]/r.dens + r.dens*r.d2fdxs2[is]*dxsdn[s1]*dxsdn[s2];
